/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "fileloadhelper.h"

#include <kdebug.h>
#include <ksavefile.h>
#include <ktempfile.h>
#include <kio/netaccess.h>

FileLoadHelper::FileLoadHelper( const KURL& url ):
	m_url( url ),
	m_file( 0 )
{
}

FileLoadHelper::~FileLoadHelper()
{
	if ( m_file )
		close();
}

const KURL& FileLoadHelper::url()
{
	return m_url;
}

QFile* FileLoadHelper::file()
{
	return m_file;
}

bool FileLoadHelper::open()
{
	if ( m_file )
		return false;

	if ( ! KIO::NetAccess::exists( m_url, true, 0 ) )
		return false;

	if ( m_url.isLocalFile() )
	{
		m_file = new QFile( m_url.path() );
		if ( ! m_file->open( IO_ReadOnly ) )
		{
			kdDebug() << "couldn't open input file " << m_file->name() << endl;
			delete m_file;
			m_file = 0;
			return false;
		}
	}
	else
	{
		QString tmpFile;
		if ( ! KIO::NetAccess::download( m_url, tmpFile, 0 ) )
		{
			kdDebug() << "couldn't get input url: " << m_url.prettyURL() << endl;
			kdDebug() << KIO::NetAccess::lastErrorString() << endl;
			return false;
		}

		m_file = new QFile( tmpFile );
		if ( ! m_file->open( IO_ReadOnly ) )
		{
			kdDebug() << "couldn't open input file " << m_file->name() << endl;
			delete m_file;
			m_file = 0;
			return false;
		}
	}
	return true;
}

bool FileLoadHelper::close()
{
	if ( ! m_file )
		return false;

	QString tmpFilePath = m_file->name();

	delete m_file; // closes the file
	m_file = 0;

	if ( ! m_url.isLocalFile() )
		KIO::NetAccess::removeTempFile( tmpFilePath );

	return true;
}

bool FileLoadHelper::exists( const KURL& url )
{
	return KIO::NetAccess::exists( url, true, 0 );
}
