/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "filesavehelper.h"

#include <kdebug.h>
#include <ktempfile.h>
#include <kio/netaccess.h>

FileSaveHelper::FileSaveHelper( const KURL& url, bool overwrite ):
	m_url( url ),
	m_overwrite( overwrite ),
	m_file( 0 ),
	m_tmpFile( 0 )
{
}

FileSaveHelper::~FileSaveHelper()
{
	if ( m_file )
		close();
}

const KURL& FileSaveHelper::url()
{
	return m_url;
}

bool FileSaveHelper::overwrite()
{
	return m_overwrite;
}

QFile* FileSaveHelper::file()
{
	return m_file;
}

bool FileSaveHelper::open()
{
	if ( m_file )
		return false;

	if ( ! m_overwrite && KIO::NetAccess::exists( m_url, false, 0 ) )
		return false;

	if ( m_url.isLocalFile() )
	{
		m_file = new QFile( m_url.path() );
		if ( ! m_file->open( IO_WriteOnly|IO_Truncate ) )
		{
			kdDebug() << "couldn't open output file " << m_file->name() << endl;
			delete m_file;
			m_file = 0;
			return false;
		}
	}
	else
	{
		m_tmpFile = new KTempFile();
		m_file = m_tmpFile->file();
		if ( ! m_file )
		{
			kdDebug() << "couldn't open output file " << m_tmpFile->name() << endl;
			delete m_tmpFile;
			m_tmpFile = 0;
			m_file = 0;
			return false;
		}
	}
	return true;
}

bool FileSaveHelper::close()
{
	if ( ! m_file )
		return false;

	if ( m_url.isLocalFile() )
	{
		delete m_file; // the destructor calls close()
		m_file = 0;
		return true;
	}
	else
	{
		m_tmpFile->close(); // close the file to ensure everything has been written to it

		bool success =
			m_overwrite ?
			KIO::NetAccess::upload( m_tmpFile->name(), m_url, 0 ) :
			KIO::NetAccess::file_copy( KURL( m_tmpFile->name() ), m_url, 0 );

		delete m_tmpFile; // the destructor removes the temporary file
		m_tmpFile = 0;
		m_file = 0;

		return success;
	}
}

bool FileSaveHelper::exists( const KURL& url )
{
	return KIO::NetAccess::exists( url, false, 0 );
}
