#ifndef RANGE_H
#define RANGE_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <qvaluevector.h>

#include <kdebug.h>

namespace SubtitleComposer
{
	class Range
	{
		public:

			static const unsigned MaxIndex=(unsigned)-1;

			Range( unsigned index ):
				m_start( index ),
				m_end( index )
			{
				m_start = index;
				m_end = index;
			}

			Range( unsigned startIndex, unsigned endIndex ):
				m_start( startIndex ),
				m_end( endIndex )
			{
				if ( m_start > m_end )
				{
					kdWarning() << "Attempted to create invalid range: " << startIndex << ";" << endIndex << endl;
					m_start = m_end;
				}
			}

			Range( const Range& range ):
				m_start( range.m_start ),
				m_end( range.m_end )
			{
			}

			Range& operator=( const Range& range )
			{
				if ( this == &range )
					return *this;

				m_start = range.m_start;
				m_end = range.m_end;

				return *this;
			}

			inline static Range full()
			{
				return Range( 0, MaxIndex );
			}

			inline static Range lower( unsigned index )
			{
				return Range( 0, index );
			}

			inline static Range upper( unsigned index )
			{
				return Range( index, MaxIndex );
			}

			inline unsigned start() const { return m_start; }
			inline unsigned end() const { return m_end; }

			inline bool contains( unsigned index ) const
			{
				return index >= m_start && index <= m_end;
			}

			inline bool contains( const Range& range ) const
			{
				return m_end <= range.m_end && m_start >= range.m_start;
			}

			inline bool operator==( const Range& range ) const
			{
				return m_start == range.m_start && m_end == range.m_end;
			}

			inline bool operator!=( const Range& range ) const
			{
				return m_start != range.m_start || m_end != range.m_end;
			}

			inline bool operator<( const Range& range ) const
			{
				return m_start < range.m_end;
			}

			inline bool operator<=( const Range& range ) const
			{
				return m_start <= range.m_end;
			}

			inline bool operator>( const Range& range ) const
			{
				return m_end > range.m_start;
			}

			inline bool operator>=( const Range& range ) const
			{
				return m_end >= range.m_start;
			}

		private:

			friend class RangeList;
			friend class QValueVector<Range>;
			friend class QValueVectorPrivate<Range>;

			Range() {}

			unsigned m_start;
			unsigned m_end;
	};
}

#endif
