#ifndef SUBRIPOUTPUTFORMAT_H
#define SUBRIPOUTPUTFORMAT_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <outputformat.h>

namespace SubtitleComposer
{
	class SubRipOutputFormat : public OutputFormat
	{
		friend class FormatManager;

		public:

			~SubRipOutputFormat() {}

			virtual QString dumpSubtitles( const Subtitle& subtitle, bool primary ) const
			{
				QString ret;

				for ( SubtitleIterator it( subtitle ); it.current(); ++it )
				{
					const SubtitleLine* line = it.current();

					Time showTime = line->showTime();
					Time hideTime = line->hideTime();
					ret += m_timeBuilder.sprintf(
						"%d\n%02d:%02d:%02d,%03d --> %02d:%02d:%02d,%03d\n",
						it.index() + 1,
						showTime.hours(), showTime.minutes(), showTime.seconds(), showTime.mseconds(),
						hideTime.hours(), hideTime.minutes(), hideTime.seconds(), hideTime.mseconds()
					);

					const SString& text = primary ? line->primaryText() : line->secondaryText();
					ret += text.richString();

					ret += "\n\n";
				}

				return ret;
			}

		protected:

			SubRipOutputFormat():
				OutputFormat( "SubRip", "srt" ),
				m_dialogueBuilder( "%1%2%3%4%5%6%7\n\n" )
			{
			}

			const QString m_dialogueBuilder;
			mutable QString m_timeBuilder;
	};
}

#endif
