/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "playerconfigwidget.h"
#include "playerconfigdefs.h"
#include <player.h>

#include <layeredwidget.h>
#include <textoverlaywidget.h>

#include <qlayout.h>
#include <qpalette.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qcheckbox.h>

#include <kcombobox.h>
#include <kcolorcombo.h>
#include <kfontcombo.h>
#include <knuminput.h>

#include <klocale.h>

using namespace SubtitleComposer;

PlayerConfigWidget::PlayerConfigWidget( const AppConfigGroup& configGroup, QWidget* parent ):
	AppConfigGroupWidget( configGroup, parent )
{
	QGroupBox* playerGroupBox = createGroupBox( i18n( "General" ) );

	QLabel* playerBackendLabel = new QLabel( playerGroupBox );
	playerBackendLabel->setText( i18n( "Player backend" ) );

	m_playerBackendComboBox = new KComboBox( playerGroupBox );
	m_playerBackendComboBox->insertStringList( Player::instance()->backendNames() );

	QLabel* seekJumpLabel = new QLabel( playerGroupBox );
	seekJumpLabel->setText( i18n( "Jump length on forwards/backwards seek" ) );

	m_seekJumpSecsSpinBox = new KIntNumInput( playerGroupBox );
	m_seekJumpSecsSpinBox->setMinValue( 1 );
	m_seekJumpSecsSpinBox->setMaxValue( 600 );
	m_seekJumpSecsSpinBox->setSuffix( i18n( " secs" ) );


	QGroupBox* fontGroupBox = createGroupBox( i18n( "Subtitles font" ) );

	QLabel* fontLabel = new QLabel( fontGroupBox );
	fontLabel->setText( i18n( "Family" ) );

	m_fontComboBox = new KFontCombo( fontGroupBox );

	QLabel* sizeLabel = new QLabel( fontGroupBox );
	sizeLabel->setText( i18n( "Size" ) );

	m_sizeSpinBox = new KIntNumInput( fontGroupBox );
	m_sizeSpinBox->setMinValue( 4 );
	m_sizeSpinBox->setMaxValue( 100 );
	m_sizeSpinBox->setSuffix( i18n( " points" ) );

	QLabel* primaryColorLabel = new QLabel( fontGroupBox );
	primaryColorLabel->setText( i18n( "Primary Color" ) );

	m_primaryColorComboBox = new KColorCombo( fontGroupBox );

	QLabel* outlineColorLabel = new QLabel( fontGroupBox );
	outlineColorLabel->setText( i18n( "Outline Color" ) );

	m_outlineColorComboBox = new KColorCombo( fontGroupBox );


	QLabel* outlineWidthLabel = new QLabel( fontGroupBox );
	outlineWidthLabel->setText( i18n( "Outline Width" ) );

	m_outlineWidthSpinBox = new KIntNumInput( fontGroupBox );
	m_outlineWidthSpinBox->setMinValue( 0 );
	m_outlineWidthSpinBox->setMaxValue( 5 );
	m_outlineWidthSpinBox->setSuffix( i18n( " pixels" ) );


	QGroupBox* previewGroupBox = createGroupBox( i18n( "Subtitles preview" ) );

	QWidget* previewWidget = new LayeredWidget( previewGroupBox );
// 	previewWidget->setBackgroundColor( Qt::black );
	previewWidget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
	previewWidget->setMinimumHeight( 60 );

	m_textOverlayWidget = new TextOverlayWidget( previewWidget );
	m_textOverlayWidget->setText( i18n( "<p>The Quick Brown Fox<br/>Jumps Over The Lazy Dog</p>" ) );
	m_textOverlayWidget->setRichTextMode( true );
	m_textOverlayWidget->setOutlineWidth( 1 );
	m_textOverlayWidget->setAlignment( Qt::AlignHCenter|Qt::AlignVCenter );

	QGridLayout* playerLayout = createGridLayout( playerGroupBox );
	playerLayout->addWidget( playerBackendLabel, 0, 0 );
	playerLayout->addWidget( m_playerBackendComboBox, 0, 1 );
	playerLayout->addWidget( seekJumpLabel, 1, 0 );
	playerLayout->addWidget( m_seekJumpSecsSpinBox, 1, 1 );

	QGridLayout* fontLayout = createGridLayout( fontGroupBox );
	fontLayout->addWidget( fontLabel, 0, 0 );
	fontLayout->addWidget( m_fontComboBox, 0, 1 );
	fontLayout->addWidget( sizeLabel, 1, 0 );
	fontLayout->addWidget( m_sizeSpinBox, 1, 1 );
	fontLayout->addWidget( primaryColorLabel, 2, 0 );
	fontLayout->addWidget( m_primaryColorComboBox, 2, 1 );
	fontLayout->addWidget( outlineColorLabel, 3, 0 );
	fontLayout->addWidget( m_outlineColorComboBox, 3, 1 );
	fontLayout->addWidget( outlineWidthLabel, 4, 0 );
	fontLayout->addWidget( m_outlineWidthSpinBox, 4, 1 );


	QGridLayout* previewLayout = createGridLayout( previewGroupBox );
	previewLayout->addWidget( previewWidget, 0, 0 );

	connect( m_playerBackendComboBox, SIGNAL(activated(int)), this, SIGNAL(settingsChanged()) );
	connect( m_seekJumpSecsSpinBox, SIGNAL(valueChanged(int)), this, SIGNAL(settingsChanged()) );

	connect( m_fontComboBox, SIGNAL(activated(const QString&)), this, SLOT(onFamilyChanged(const QString&)) );
	connect( m_fontComboBox, SIGNAL(textChanged(const QString&)), this, SLOT(onFamilyChanged(const QString&)) );
	connect( m_sizeSpinBox, SIGNAL(valueChanged(int)), this, SLOT(onSizeChanged(int)) );
	connect( m_primaryColorComboBox, SIGNAL(activated(const QColor&)), this, SLOT(onPrimaryColorChanged(const QColor&)) );
	connect( m_outlineColorComboBox, SIGNAL(activated(const QColor&)), this, SLOT(onOutlineColorChanged(const QColor&)) );
	connect( m_outlineWidthSpinBox, SIGNAL(valueChanged(int)), this, SLOT(onOutlineWidthChanged(int)) );

	connect( m_fontComboBox, SIGNAL(activated(int)), this, SIGNAL(settingsChanged()) );
	connect( m_sizeSpinBox, SIGNAL(valueChanged(int)), this, SIGNAL(settingsChanged()) );
	connect( m_primaryColorComboBox, SIGNAL(activated(const QColor&)), this, SIGNAL(settingsChanged()) );
	connect( m_outlineColorComboBox, SIGNAL(activated(const QColor&)), this, SIGNAL(settingsChanged()) );
	connect( m_outlineWidthSpinBox, SIGNAL(valueChanged(int)), this, SIGNAL(settingsChanged()) );


	setControlsFromConfig();
}

PlayerConfigWidget::~PlayerConfigWidget()
{
}

void PlayerConfigWidget::setControlsFromConfig()
{
	m_playerBackendComboBox->setCurrentItem( m_configGroup.option( OPT_PLAYER_BACKEND ) );
	m_seekJumpSecsSpinBox->setValue( m_configGroup.optionAsInt( OPT_JUMP_ON_SEEK_SECS ) );

	QString family( m_configGroup.option( OPT_FONT_FAMILY ) );
	m_fontComboBox->setCurrentFont( family );
	onFamilyChanged( family );

	int size = m_configGroup.optionAsInt( OPT_FONT_SIZE );
	m_sizeSpinBox->setValue( size );
	onSizeChanged( size );

	QColor primaryColor( m_configGroup.optionAsColor( OPT_FONT_COLOR ) );
	m_primaryColorComboBox->setColor( primaryColor );
	onPrimaryColorChanged( primaryColor );

	QColor outlineColor( m_configGroup.optionAsColor( OPT_OUTLINE_COLOR ) );
	m_outlineColorComboBox->setColor( outlineColor );
	onOutlineColorChanged( outlineColor );

	int outlineWidth = m_configGroup.optionAsInt( OPT_OUTLINE_WIDTH );
	m_outlineWidthSpinBox->setValue( outlineWidth );
	onOutlineWidthChanged( outlineWidth );
}

void PlayerConfigWidget::setConfigFromControls()
{
	m_configGroup.setOption( OPT_PLAYER_BACKEND, m_playerBackendComboBox->currentText() );
	m_configGroup.setOption( OPT_JUMP_ON_SEEK_SECS, m_seekJumpSecsSpinBox->value() );

	m_configGroup.setOption( OPT_FONT_FAMILY, m_textOverlayWidget->family() );
	m_configGroup.setOption( OPT_FONT_SIZE, m_textOverlayWidget->pointSize() );
	m_configGroup.setOption( OPT_FONT_COLOR, m_textOverlayWidget->primaryColor() );
	m_configGroup.setOption( OPT_OUTLINE_COLOR, m_textOverlayWidget->outlineColor() );
	m_configGroup.setOption( OPT_OUTLINE_WIDTH, (int)m_textOverlayWidget->outlineWidth() );
}

void PlayerConfigWidget::onFamilyChanged( const QString& family )
{
	if ( m_fontComboBox->contains( family ) )
		m_textOverlayWidget->setFamily( family );
}

void PlayerConfigWidget::onSizeChanged( int size )
{
	m_textOverlayWidget->setPointSize( size );
}

void PlayerConfigWidget::onPrimaryColorChanged( const QColor& color )
{
	m_textOverlayWidget->setPrimaryColor( color );
}

void PlayerConfigWidget::onOutlineColorChanged( const QColor& color )
{
	m_textOverlayWidget->setOutlineColor( color );
}

void PlayerConfigWidget::onOutlineWidthChanged( int width )
{
	m_textOverlayWidget->setOutlineWidth( width );
}
