/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "currentlinewidget.h"
#include <timeedit.h>
#include <simplerichtextedit.h>
#include <qxtsignalwaiter.h>
#include <kapplication.h>

#include <kdebug.h>
#include <klocale.h>
#include <kiconloader.h>

#include <qlayout.h>
#include <qlabel.h>
#include <qtoolbutton.h>
#include <qgroupbox.h>

using namespace SubtitleComposer;

QToolButton* CurrentLineWidget::createToolButton( const QString& text, const char* icon, QObject* receiver, const char* slot )
{
	KIconLoader iconLoader;

	QToolButton* toolButton = new QToolButton( this );
	toolButton->setTextLabel( text );
	toolButton->setIconSet( iconLoader.loadIconSet( icon, KIcon::Toolbar, 14, false ) );
	toolButton->setMinimumSize( 20, 20 );
	toolButton->setMaximumSize( 20, 20 );
	toolButton->setToggleButton( true );
	toolButton->setAutoRaise( true );
	connect( toolButton, SIGNAL( clicked() ), receiver, slot );
	return toolButton;
}

CurrentLineWidget::CurrentLineWidget( QWidget* parent ):
	QWidget( parent ),
	m_currentLine( 0 ),
	m_translationMode( false ),
	m_hasFocus( false ),
	m_updateCurrentLine( true ),
	m_updateControls( true )
{
	QGroupBox* timesControlsGroupBox = new QGroupBox( this );
	timesControlsGroupBox->setColumnLayout( 0, Qt::Vertical );
	timesControlsGroupBox->layout()->setSpacing( 5 );
	timesControlsGroupBox->layout()->setMargin( 6 );

	QLabel* showTimeLabel = new QLabel( i18n( "<b>Show</b>" ), timesControlsGroupBox );
	m_showTimeEdit = new TimeEdit( timesControlsGroupBox );

	QLabel* hideTimeLabel = new QLabel( i18n( "<b>Hide</b>" ), timesControlsGroupBox );
	m_hideTimeEdit = new TimeEdit( timesControlsGroupBox );

	QLabel* durationTimeLabel = new QLabel( i18n( "<b>Duration</b>" ), timesControlsGroupBox );
	m_durationTimeEdit = new TimeEdit( timesControlsGroupBox ) ;


	m_primaryTextLabel = new QLabel( this );
	m_primaryTextLabel->setAlignment( Qt::AlignLeft|Qt::AlignVCenter|Qt::WordBreak );
	m_primaryTextLabel->setTextFormat( Qt::PlainText );
	m_primaryTextEdit = new SimpleRichTextEdit( this );
	m_primaryTextEdit->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Maximum );

	m_primaryBoldButton = createToolButton( i18n( "Toggle Bold" ), "text_bold", m_primaryTextEdit, SLOT( toggleBold() ) );
	m_primaryItalicButton = createToolButton( i18n( "Toggle Italic" ), "text_italic", m_primaryTextEdit, SLOT( toggleItalic() ) );
	m_primaryUnderlineButton = createToolButton( i18n( "Toggle Underline" ), "text_under", m_primaryTextEdit, SLOT( toggleUnderline() ) );
	m_primaryStrikeThroughButton = createToolButton( i18n( "Toggle Strike Through" ), "text_strike", m_primaryTextEdit, SLOT( toggleStrikeOut() ) );

	m_secondaryTextLabel = new QLabel( this );
	m_secondaryTextLabel->setAlignment( Qt::AlignLeft|Qt::AlignVCenter|Qt::WordBreak );
	m_secondaryTextLabel->setTextFormat( Qt::PlainText );
	m_secondaryTextEdit = new SimpleRichTextEdit( this );
	m_secondaryTextEdit->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Maximum );

	m_secondaryBoldButton = createToolButton( i18n( "Toggle Bold" ), "text_bold", m_secondaryTextEdit, SLOT( toggleBold() ) );
	m_secondaryItalicButton = createToolButton( i18n( "Toggle Italic" ), "text_italic", m_secondaryTextEdit, SLOT( toggleItalic() ) );
	m_secondaryUnderlineButton = createToolButton( i18n( "Toggle Underline" ), "text_under", m_secondaryTextEdit, SLOT( toggleUnderline() ) );
	m_secondaryStrikeThroughButton = createToolButton( i18n( "Toggle Strike Through" ), "text_strike", m_secondaryTextEdit, SLOT( toggleStrikeOut() ) );


	QFont font = m_primaryTextLabel->font();
	font.setPointSize( font.pointSize() - 1 );
	m_primaryTextLabel->setFont( font );
	m_secondaryTextLabel->setFont( font );


	QGridLayout* primaryButtonsLayout = new QGridLayout();
// 	primaryButtonsLayout->setMargin( 0 );
	primaryButtonsLayout->addWidget( m_primaryBoldButton, 0, 0, Qt::AlignBottom );
	primaryButtonsLayout->addWidget( m_primaryItalicButton, 0, 1, Qt::AlignBottom );
	primaryButtonsLayout->addWidget( m_primaryUnderlineButton, 0, 2, Qt::AlignBottom );
	primaryButtonsLayout->addWidget( m_primaryStrikeThroughButton, 0, 3, Qt::AlignBottom );

	QGridLayout* secondaryButtonsLayout = new QGridLayout();
// 	primaryButtonsLayout->setMargin( 0 );
	secondaryButtonsLayout->addWidget( m_secondaryBoldButton, 0, 0, Qt::AlignBottom );
	secondaryButtonsLayout->addWidget( m_secondaryItalicButton, 0, 1, Qt::AlignBottom );
	secondaryButtonsLayout->addWidget( m_secondaryUnderlineButton, 0, 2, Qt::AlignBottom );
	secondaryButtonsLayout->addWidget( m_secondaryStrikeThroughButton, 0, 3, Qt::AlignBottom );


	QGridLayout* timesControlsLayout = new QGridLayout( timesControlsGroupBox->layout() );
	timesControlsLayout->addWidget( showTimeLabel, 0, 0 );
	timesControlsLayout->addWidget( m_showTimeEdit, 0, 1 );
	timesControlsLayout->addWidget( hideTimeLabel, 1, 0 );
	timesControlsLayout->addWidget( m_hideTimeEdit, 1, 1 );
	timesControlsLayout->addWidget( durationTimeLabel, 2, 0 );
	timesControlsLayout->addWidget( m_durationTimeEdit, 2, 1 );

	m_mainLayout = new QGridLayout( this );
	m_mainLayout->setMargin( 0 );
	m_mainLayout->setSpacing( 0 );
	m_mainLayout->setColSpacing( 1, 5 );
	m_mainLayout->setColSpacing( 4, 0 );

	m_mainLayout->setColStretch( 2, 1 );

	m_mainLayout->addMultiCellWidget( timesControlsGroupBox, 0, 1, 0, 0 );
	m_mainLayout->addWidget( m_primaryTextLabel, 0, 2 );
	m_mainLayout->addLayout( primaryButtonsLayout, 0, 3 );
	m_mainLayout->addMultiCellWidget( m_primaryTextEdit, 1, 1, 2, 3 );
	m_mainLayout->addWidget( m_secondaryTextLabel, 0, 5 );
	m_mainLayout->addLayout( secondaryButtonsLayout, 0, 6 );
	m_mainLayout->addMultiCellWidget( m_secondaryTextEdit, 1, 1, 5, 6 );

	int maxTextHeight = timesControlsGroupBox->minimumSizeHint().height() - m_primaryBoldButton->minimumSizeHint().height() + 4;
	m_primaryTextEdit->setMaximumHeight( maxTextHeight );
	m_secondaryTextEdit->setMaximumHeight( maxTextHeight );

	timesControlsGroupBox->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
	m_primaryTextEdit->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Expanding );
	m_secondaryTextEdit->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Expanding );
	setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Expanding );

	connect( m_primaryTextEdit, SIGNAL( selectionChanged2() ), this, SLOT( onPrimaryTextEditSelectionChanged() ) );
	connect( m_primaryTextEdit, SIGNAL( cursorChanged2() ), this, SLOT( onPrimaryTextEditSelectionChanged() ) );
	connect( m_secondaryTextEdit, SIGNAL( selectionChanged2() ), this, SLOT( onSecondaryTextEditSelectionChanged() ) );
	connect( m_secondaryTextEdit, SIGNAL( cursorChanged2() ), this, SLOT( onSecondaryTextEditSelectionChanged() ) );
	connect( m_primaryTextEdit, SIGNAL( textChanged() ), this, SLOT( onPrimaryTextEditChanged() ) );
	connect( m_secondaryTextEdit, SIGNAL( textChanged() ), this, SLOT( onSecondaryTextEditChanged() ) );
	connect( m_showTimeEdit, SIGNAL( valueChanged(int) ), this, SLOT( onShowTimeEditChanged(int) ) );
	connect( m_hideTimeEdit, SIGNAL( valueChanged(int) ), this, SLOT( onHideTimeEditChanged(int) ) );
	connect( m_durationTimeEdit, SIGNAL( valueChanged(int) ), this, SLOT( onDurationTimeEditChanged(int) ) );

	setTranslationMode( m_translationMode );


	connect( m_primaryTextEdit, SIGNAL( focusChanged(bool) ), this, SLOT( onFocusChanged() ) );
	connect( m_secondaryTextEdit, SIGNAL( focusChanged(bool) ), this, SLOT( onFocusChanged() ) );
}

CurrentLineWidget::~CurrentLineWidget()
{
}

bool CurrentLineWidget::hasFocus() const
{
	return m_hasFocus;
}

void CurrentLineWidget::loadConfig()
{
}

void CurrentLineWidget::saveConfig()
{
}

void CurrentLineWidget::setSubtitle( Subtitle* subtitle )
{
	if ( ! subtitle )
		setCurrentLine( 0 );
}

void CurrentLineWidget::setCurrentLine( SubtitleLine* line )
{
	if ( m_currentLine )
	{
		disconnect( m_currentLine, SIGNAL( primaryTextChanged( const SString& ) ),
					this, SLOT( onLinePrimaryTextChanged( const SString& ) ) );
		disconnect( m_currentLine, SIGNAL( secondaryTextChanged( const SString& ) ),
					this, SLOT( onLineSecondaryTextChanged( const SString& ) ) );
		disconnect( m_currentLine, SIGNAL( showTimeChanged( const Time& ) ),
					this, SLOT( onLineShowTimeChanged( const Time& ) ) );
		disconnect( m_currentLine, SIGNAL( hideTimeChanged( const Time& ) ),
					this, SLOT( onLineHideTimeChanged( const Time& ) ) );
	}

	m_currentLine = line;

	if ( m_currentLine )
	{
		connect( m_currentLine, SIGNAL( primaryTextChanged( const SString& ) ),
				 this, SLOT( onLinePrimaryTextChanged( const SString& ) ) );
		connect( m_currentLine, SIGNAL( secondaryTextChanged( const SString& ) ),
				 this, SLOT( onLineSecondaryTextChanged( const SString& ) ) );
		connect( m_currentLine, SIGNAL( showTimeChanged( const Time& ) ),
				 this, SLOT( onLineShowTimeChanged( const Time& ) ) );
		connect( m_currentLine, SIGNAL( hideTimeChanged( const Time& ) ),
				 this, SLOT( onLineHideTimeChanged( const Time& ) ) );
	}

	onLineShowTimeChanged( m_currentLine ? m_currentLine->showTime() : 0 );
	onLineHideTimeChanged( m_currentLine ? m_currentLine->hideTime() : 0 );

	if ( m_currentLine )
	{
		onLinePrimaryTextChanged( m_currentLine->primaryText() );
		onLineSecondaryTextChanged( m_currentLine->secondaryText() );
	}
	else
	{
		m_updateCurrentLine = false;

		m_primaryTextLabel->setText( i18n( "No current line" ) );
		m_primaryTextEdit->setRichText( SString() );
		onPrimaryTextEditSelectionChanged();

		if ( m_translationMode )
		{
			m_secondaryTextLabel->setText( i18n( "No current line" ) );
			m_secondaryTextEdit->setRichText( SString() );
			onSecondaryTextEditSelectionChanged();
		}

		m_updateCurrentLine = true;
	}

	setEnabled( m_currentLine != 0 );
}

void CurrentLineWidget::setTranslationMode( bool enabled )
{
	m_translationMode = enabled;

	if ( m_translationMode )
	{
		m_secondaryTextLabel->show();
		m_secondaryTextEdit->show();

		m_secondaryBoldButton->show();
		m_secondaryItalicButton->show();
		m_secondaryUnderlineButton->show();
		m_secondaryStrikeThroughButton->show();

		m_mainLayout->setColSpacing( 4, 5 );
		m_mainLayout->setColStretch( 5, 1 );
	}
	else
	{
		m_secondaryTextLabel->hide();
		m_secondaryTextEdit->hide();

		m_secondaryBoldButton->hide();
		m_secondaryItalicButton->hide();
		m_secondaryUnderlineButton->hide();
		m_secondaryStrikeThroughButton->hide();

		m_mainLayout->setColSpacing( 4, 0 );
		m_mainLayout->setColStretch( 5, 0 );
	}

	setCurrentLine( m_currentLine );
}

void CurrentLineWidget::onPrimaryTextEditSelectionChanged()
{
	m_primaryBoldButton->setOn( m_primaryTextEdit->bold() );
	m_primaryItalicButton->setOn( m_primaryTextEdit->italic() );
	m_primaryUnderlineButton->setOn( m_primaryTextEdit->underline() );
	m_primaryStrikeThroughButton->setOn( m_primaryTextEdit->strikeOut() );
}

void CurrentLineWidget::onSecondaryTextEditSelectionChanged()
{
	m_secondaryBoldButton->setOn( m_secondaryTextEdit->bold() );
	m_secondaryItalicButton->setOn( m_secondaryTextEdit->italic() );
	m_secondaryUnderlineButton->setOn( m_secondaryTextEdit->underline() );
	m_secondaryStrikeThroughButton->setOn( m_secondaryTextEdit->strikeOut() );
}

void CurrentLineWidget::onPrimaryTextEditChanged()
{
	if ( m_updateCurrentLine )
	{
		m_updateControls = false;

		SString text( m_primaryTextEdit->richText() );
		m_currentLine->setPrimaryText( text );
		m_primaryTextLabel->setText( buildTextDescription( text.string() ) );

		m_updateControls = true;
	}
}

void CurrentLineWidget::onSecondaryTextEditChanged()
{
	if ( m_updateCurrentLine )
	{
		m_updateControls = false;

		SString text( m_secondaryTextEdit->richText() );
		m_currentLine->setSecondaryText( text );
		m_secondaryTextLabel->setText( buildTextDescription( text.string() ) );

		m_updateControls = true;
	}
}

void CurrentLineWidget::onShowTimeEditChanged( int showTime )
{
	if ( m_updateCurrentLine )
	{
		m_updateControls = false;
		m_currentLine->setShowTime( showTime );
		m_durationTimeEdit->setValue( m_hideTimeEdit->value() - showTime );
		m_updateControls = true;
	}
}

void CurrentLineWidget::onHideTimeEditChanged( int hideTime )
{
	if ( m_updateCurrentLine )
	{
		m_updateControls = false;
		m_currentLine->setHideTime( hideTime );
		m_durationTimeEdit->setValue( hideTime - m_showTimeEdit->value() );
		m_updateControls = true;
	}
}

void CurrentLineWidget::onDurationTimeEditChanged( int durationTime )
{
	if ( m_updateCurrentLine )
	{
		m_updateControls = false;
		m_currentLine->setDurationTime( durationTime );
		m_hideTimeEdit->setValue( m_showTimeEdit->value() + durationTime );
		m_updateControls = true;
	}
}

QString CurrentLineWidget::buildTextDescription( const QString& text )
{
	// we directly use this version of i18n (instead of the one for numbers)
	// for (presumed) performance reasons (don't want to go check the dict
	// for every key pressed).

	static QString loadedText( i18n( "Current line: %1 characters" ) );

	static QString plus( " + " );
	static QString equals( " = " );

	QStringList lines( QStringList::split( "\n", text, true ) );

	QString characters;
	int totalChars = 0, lineChars;

	for ( int index = 0, lastIndex = lines.size()-2; index <= lastIndex; ++index )
	{
		lineChars = lines[index].length();
		totalChars += lineChars;
		characters += QString::number( lineChars ) + plus;
	}

	if ( lines.size() > 1 )
	{
		lineChars = lines.last().length();
		totalChars += lineChars;
		characters += QString::number( lineChars ) + equals + QString::number( totalChars );
	}
	else
	{
		if ( lines.size() )
			totalChars += lines.last().length();
		characters = QString::number( totalChars );
	}

	return QString( loadedText ).arg( characters );
}

void CurrentLineWidget::onLinePrimaryTextChanged( const SString& primaryText )
{
	if ( m_updateControls )
	{
		m_updateCurrentLine = false;

		m_primaryTextLabel->setText( buildTextDescription( primaryText.string() ) );
		m_primaryTextEdit->setRichText( primaryText );

		onPrimaryTextEditSelectionChanged();

		m_updateCurrentLine = true;
	}
}

void CurrentLineWidget::onLineSecondaryTextChanged( const SString& secondaryText )
{
	if ( m_updateControls && m_translationMode )
	{
		m_updateCurrentLine = false;

		m_secondaryTextLabel->setText( buildTextDescription( secondaryText.string() ) );
		m_secondaryTextEdit->setRichText( secondaryText );

		onSecondaryTextEditSelectionChanged();

		m_updateCurrentLine = true;
	}
}

void CurrentLineWidget::onLineShowTimeChanged( const Time& showTime )
{
	if ( m_updateControls )
	{
		m_updateCurrentLine = false;
		m_showTimeEdit->setValue( showTime.toMillis() );
		m_updateCurrentLine = true;
	}
}

void CurrentLineWidget::onLineHideTimeChanged( const Time& hideTime )
{
	if ( m_updateControls )
	{
		m_updateCurrentLine = false;
		m_hideTimeEdit->setValue( hideTime.toMillis() );
		m_durationTimeEdit->setValue( m_hideTimeEdit->value() - m_showTimeEdit->value() );
		m_updateCurrentLine = true;
	}
}

void CurrentLineWidget::focusInEvent( QFocusEvent* event )
{
	QWidget::focusInEvent( event );

	onFocusChanged();
}

void CurrentLineWidget::focusOutEvent( QFocusEvent* event )
{
	QWidget::focusOutEvent( event );

	onFocusChanged();
}

void CurrentLineWidget::onFocusChanged()
{
	QWidget* focusWidget = kapp->focusWidget();

	bool hasFocus = focusWidget == this || focusWidget == m_primaryTextEdit || focusWidget == m_secondaryTextEdit;
	if ( m_hasFocus != hasFocus )
	{
		m_hasFocus = hasFocus;
		emit focusChanged( hasFocus );
	}
}

void CurrentLineWidget::highlightPrimary( int startIndex, int endIndex )
{
	m_primaryTextEdit->setSelection( startIndex, endIndex );
}

void CurrentLineWidget::highlightSecondary( int startIndex, int endIndex )
{
	m_secondaryTextEdit->setSelection( startIndex, endIndex );
}

#include "currentlinewidget.moc"
