/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "simplerichtextedit.h"
#include "sstring.h"

#include <qpopupmenu.h>
#include <qregexp.h>
#include <qstylesheet.h>

#include <kdebug.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kspell.h>
#include <kaccel.h>

#define SUBTITLECOMPOSER
#ifdef SUBTITLECOMPOSER
	#include "../main/useractions/useractionnames.h"
	#include <qkeysequence.h>
	#include <kapplication.h>
	#include <kmainwindow.h>
	#include <kactioncollection.h>
	#include <kaction.h>
#endif

SimpleRichTextEdit::SimpleRichTextEdit( QWidget* parent ):
	QTextEdit( parent, 0 ),
	m_emitSignals( true )
{
	QTextEdit::setTextFormat( Qt::PlainText );

	setAutoFormatting( QTextEdit::AutoNone );
	setWordWrap( QTextEdit::WidgetWidth );

	QStyleSheet* styleSheet = new QStyleSheet( this );
	setStyleSheet( styleSheet );

	QStyleSheetItem* pItem = new QStyleSheetItem( styleSheet, "p" );
	pItem->setDisplayMode( QStyleSheetItem::DisplayBlock );
	pItem->setMargin( QStyleSheetItem::MarginVertical, 0 );
	pItem->setAlignment( Qt::AlignCenter );
	pItem->setFontSize( font().pointSize() + 2 );
	pItem->setWhiteSpaceMode( QStyleSheetItem::WhiteSpacePre );

	connect( this, SIGNAL( selectionChanged() ), this, SLOT( onSelectionChanged() ) );
	connect( this, SIGNAL( cursorPositionChanged(int,int) ), this, SLOT( onCursorPositionChanged() ) );
}

SimpleRichTextEdit::~SimpleRichTextEdit()
{
}

void SimpleRichTextEdit::setTextFormat( TextFormat /*format*/ )
{
}

SubtitleComposer::SString SimpleRichTextEdit::richText()
{
// 	kdDebug() << "setting rich text" << endl;

	SubtitleComposer::SString richText( text() );

	setUpdatesEnabled( false );

	if ( richText.length() )
	{
		int origPar, origParIndex;
		getCursorPosition( &origPar, &origParIndex );

		int styleFlags;

		unsigned richTextIndex = 0;
		for ( unsigned par = 0; par < (unsigned)paragraphs(); ++par )
		{
			for ( unsigned parIndex = 0, parSize = paragraphLength( par ); parIndex < parSize; ++parIndex )
			{
				setCursorPosition( par, parIndex + 1, false );

				styleFlags = 0;
				if ( bold() )			styleFlags |= SubtitleComposer::SString::Bold;
				if ( italic() )			styleFlags |= SubtitleComposer::SString::Italic;
				if ( underline() )		styleFlags |= SubtitleComposer::SString::Underline;
				if ( strikeOut() )		styleFlags |= SubtitleComposer::SString::StrikeThrough;

				richText.setStyleFlagsAt( richTextIndex++, styleFlags );
			}

			setCursorPosition( par + 1, 0, false );

			styleFlags = 0;
			if ( bold() )			styleFlags |= SubtitleComposer::SString::Bold;
			if ( italic() )			styleFlags |= SubtitleComposer::SString::Italic;
			if ( underline() )		styleFlags |= SubtitleComposer::SString::Underline;
			if ( strikeOut() )		styleFlags |= SubtitleComposer::SString::StrikeThrough;

			richText.setStyleFlagsAt( richTextIndex++, styleFlags ); // use the previous char style for new lines
		}

		setCursorPosition( origPar, origParIndex, false );
	}

	setUpdatesEnabled( true );

	return richText;
}


void SimpleRichTextEdit::setRichText( const SubtitleComposer::SString& richText )
{
	QTextEdit::clear();

	setUpdatesEnabled( false );

	QString text( richText.string() );

	int parStart = 0, parEnd, par = 0;
	while ( (parEnd = text.find( '\n', parStart )) != -1 )
	{
		setAlignment( Qt::AlignCenter );
		setPointSize( font().pointSize() + 2 );
		if ( parEnd > parStart )
		{
			append( text.mid( parStart, parEnd - parStart ) );

			int sectionStart = parStart, sectionEnd = parStart + 1;
			int currentStyleFlags = richText.styleFlagsAt( sectionStart );
			for ( ; sectionEnd <= parEnd; ++sectionEnd )
			{
				if ( currentStyleFlags != richText.styleFlagsAt( sectionEnd ) )
				{
					setSelection( par, sectionStart - parStart, par, sectionEnd - parStart, 0, false );
					setBold( currentStyleFlags & SubtitleComposer::SString::Bold );
					setItalic( currentStyleFlags & SubtitleComposer::SString::Italic );
					setUnderline( currentStyleFlags & SubtitleComposer::SString::Underline );
					setStrikeOut( currentStyleFlags & SubtitleComposer::SString::StrikeThrough );
					sectionStart = sectionEnd;
					currentStyleFlags = richText.styleFlagsAt( sectionEnd );
				}
			}

			setSelection( par, sectionStart - parStart, par, sectionEnd - parStart, 0, false );
			setBold( currentStyleFlags & SubtitleComposer::SString::Bold );
			setItalic( currentStyleFlags & SubtitleComposer::SString::Italic );
			setUnderline( currentStyleFlags & SubtitleComposer::SString::Underline );
			setStrikeOut( currentStyleFlags & SubtitleComposer::SString::StrikeThrough );
		}
		else
			insertParagraph( QString::null, par );

		parStart = parEnd + 1;
		par++;
	}

	setAlignment( Qt::AlignCenter  );
	setPointSize( font().pointSize() + 2 );
	append( text.mid( parStart ) );

	if ( parStart < (signed)text.length() )
	{
		int sectionStart = parStart, sectionEnd = parStart + 1;
		int currentStyleFlags = richText.styleFlagsAt( sectionStart );
		for ( int size = text.length(); sectionEnd < size; ++sectionEnd )
		{
			if ( currentStyleFlags != richText.styleFlagsAt( sectionEnd ) )
			{
				setSelection( par, sectionStart - parStart, par, sectionEnd - parStart, 0, false );
				setBold( currentStyleFlags & SubtitleComposer::SString::Bold );
				setItalic( currentStyleFlags & SubtitleComposer::SString::Italic );
				setUnderline( currentStyleFlags & SubtitleComposer::SString::Underline );
				setStrikeOut( currentStyleFlags & SubtitleComposer::SString::StrikeThrough );
				sectionStart = sectionEnd;
				currentStyleFlags = richText.styleFlagsAt( sectionEnd );
			}
		}

		setSelection( par, sectionStart - parStart, par, sectionEnd - parStart, 0, false );
		setBold( currentStyleFlags & SubtitleComposer::SString::Bold );
		setItalic( currentStyleFlags & SubtitleComposer::SString::Italic );
		setUnderline( currentStyleFlags & SubtitleComposer::SString::Underline );
		setStrikeOut( currentStyleFlags & SubtitleComposer::SString::StrikeThrough );
	}

	QTextEdit::removeSelection();

	setUpdatesEnabled( true );

	update();
}

void SimpleRichTextEdit::toggleItalic()
{
	setItalic( ! italic() );
}

void SimpleRichTextEdit::toggleBold()
{
	setBold( ! bold() );
}

void SimpleRichTextEdit::toggleUnderline()
{
	setUnderline( ! underline() );
}

bool SimpleRichTextEdit::strikeOut()
{
	return currentFont().strikeOut();
}

void SimpleRichTextEdit::setStrikeOut( bool enabled )
{
	// TODO this should iterate the selected text adding the strike out format and setting the currentFont

	QFont currentFont( this->currentFont() );
	currentFont.setStrikeOut( enabled );
	setCurrentFont( currentFont );
}

void SimpleRichTextEdit::toggleStrikeOut()
{
	QFont currentFont( this->currentFont() );
	currentFont.setStrikeOut( ! currentFont.strikeOut() );
	setCurrentFont( currentFont );
}

void SimpleRichTextEdit::toggleTabChangesFocus()
{
	setTabChangesFocus( ! tabChangesFocus() );
}

void SimpleRichTextEdit::setCursorPosition( int para, int index, bool emitSignal )
{
	m_emitSignals = emitSignal;

	QTextEdit::setCursorPosition( para, index );

	m_emitSignals = true;
}

void SimpleRichTextEdit::onCursorPositionChanged()
{
	if ( m_emitSignals )
		emit cursorChanged2();
}

void SimpleRichTextEdit::setSelection( int startIndex, int endIndex )
{
	selectAll( false );

	if ( startIndex > endIndex )
		return;

	int startPar = -1, endPar = -1;
	int parStartIndex = 0, parEndIndex;

	const int parCount = this->paragraphs();
	for ( int par = 0; par < parCount; ++par, parStartIndex = parEndIndex + 1 )
	{
		parEndIndex = parStartIndex + paragraphLength( par );

		if ( startPar < 0 && startIndex <= parEndIndex )
		{
			startIndex -= parStartIndex;
			startPar = par;
		}

		if ( endPar < 0 && endIndex <= parEndIndex )
		{
			endIndex -= parStartIndex;
			endPar = par;
		}

		if ( startPar >= 0 && endPar >= 0 )
			break;
	}

	QTextEdit::setSelection( startPar, startIndex, endPar, endIndex + 1 );
}

void SimpleRichTextEdit::setSelection( int paraFrom, int indexFrom, int paraTo, int indexTo, int selNum, bool emitSignal )
{
	m_emitSignals = emitSignal;

	QTextEdit::setSelection( paraFrom, indexFrom, paraTo, indexTo, selNum );

	m_emitSignals = true;
}

void SimpleRichTextEdit::onSelectionChanged()
{
	if ( m_emitSignals )
		emit selectionChanged2();
}


KShortcut SimpleRichTextEdit::getKShortcut( SubtitleComposer::SString::StyleFlag style )
{
#ifdef SUBTITLECOMPOSER
	KActionCollection* actionCollection = static_cast<KMainWindow*>( topLevelWidget() )->actionCollection();
	switch ( style )
	{
		case SubtitleComposer::SString::Bold:
			return actionCollection->action( ACT_TOGGLE_SELECTED_LINES_BOLD )->shortcut();
		case SubtitleComposer::SString::Italic:
			return actionCollection->action( ACT_TOGGLE_SELECTED_LINES_ITALIC )->shortcut();
		case SubtitleComposer::SString::Underline:
			return actionCollection->action( ACT_TOGGLE_SELECTED_LINES_UNDERLINE )->shortcut();
		case SubtitleComposer::SString::StrikeThrough:
			return actionCollection->action( ACT_TOGGLE_SELECTED_LINES_STRIKETHROUGH )->shortcut();
		default:
			return KShortcut();
	}
#else
	switch ( style )
	{
		case SString::Bold:
			return QKeySequence::Bold;
		case SString::Italic:
			return QKeySequence::Italic;
		case SString::Underline:
			return QKeySequence::Underline;
		case SString::StrikeThrough:
		default:
			return KShortcut();
	}
#endif
}

QKeySequence SimpleRichTextEdit::getQKeySequence( SubtitleComposer::SString::StyleFlag style )
{
	return getKShortcut( style );
}

/// next method is (mostly) taken shamelessly from KTextEdit

QPopupMenu* SimpleRichTextEdit::createPopupMenu( const QPoint& pos )
{
	enum { IdUndo, IdRedo, IdSep1, IdCut, IdCopy, IdPaste, IdClear, IdSep2, IdSelectAll };

	QPopupMenu* menu = QTextEdit::createPopupMenu( pos );

	if ( isReadOnly() )
		menu->changeItem( menu->idAt( 0 ), SmallIconSet( "editcopy" ), menu->text( menu->idAt( 0 ) ) );
	else
	{
		int id = menu->idAt( 0 );

		if ( isUndoRedoEnabled() )
		{
			menu->changeItem( id - IdUndo, SmallIconSet( "undo" ), menu->text( id - IdUndo) );
			menu->changeItem( id - IdRedo, SmallIconSet( "redo" ), menu->text( id - IdRedo) );
		}
		else
		{
			menu->removeItem( id - IdUndo );
			menu->removeItem( id - IdRedo );
			menu->removeItem( id - IdSep1 );
		}

		menu->changeItem( id - IdCut, SmallIconSet( "editcut" ), menu->text( id - IdCut ) );
		menu->changeItem( id - IdCopy, SmallIconSet( "editcopy" ), menu->text( id - IdCopy ) );
		menu->changeItem( id - IdPaste, SmallIconSet( "editpaste" ), menu->text( id - IdPaste ) );
		menu->changeItem( id - IdClear, SmallIconSet( "editclear" ), menu->text( id - IdClear ) );

		menu->insertSeparator();

		id = menu->insertItem(
			SmallIconSet( "text_bold" ), i18n( "Bold" ),
			this, SLOT( toggleBold() ),
			getQKeySequence( SubtitleComposer::SString::Bold )
		);
		menu->setItemChecked( id, bold() );

		id = menu->insertItem(
			SmallIconSet( "text_italic" ), i18n( "Italic" ),
			this, SLOT( toggleItalic() ),
			getQKeySequence( SubtitleComposer::SString::Italic )
		);
		menu->setItemChecked( id, italic() );

		id = menu->insertItem(
			SmallIconSet( "text_under" ), i18n( "Underline" ),
			this, SLOT( toggleUnderline() ),
			getQKeySequence( SubtitleComposer::SString::Underline )
		);
		menu->setItemChecked( id, underline() );

		id = menu->insertItem(
			SmallIconSet( "text_strike" ), i18n( "Strike Through" ),
			this, SLOT( toggleStrikeOut() ),
			getQKeySequence( SubtitleComposer::SString::StrikeThrough )
		);
		menu->setItemChecked( id, strikeOut() );

		menu->insertSeparator();

		id = menu->insertItem( i18n( "Allow Tabulations" ), this, SLOT( toggleTabChangesFocus() ) );
		menu->setItemChecked( id, ! tabChangesFocus() );
	}

	return menu;
}

void SimpleRichTextEdit::keyPressEvent( QKeyEvent* event )
{
	int keyValue = event->key();
	int state = event->state();
	if ( state & Qt::ControlButton )
		keyValue |= Qt::CTRL;
	if ( state & Qt::AltButton )
		keyValue |= Qt::ALT;
	if ( state & Qt::ShiftButton )
		keyValue |= Qt::SHIFT;
	if ( state & Qt::MetaButton )
		keyValue |= Qt::META;

	QKeySequence seq( keyValue );
	KKeySequence key( seq );

	if ( getKShortcut( SubtitleComposer::SString::Bold ).contains( key ) )
		toggleBold();
	else if ( getKShortcut( SubtitleComposer::SString::Italic ).contains( key ) )
		toggleItalic();
	else if ( getKShortcut( SubtitleComposer::SString::Underline ).contains( key ) )
		toggleUnderline();
	else if ( getKShortcut( SubtitleComposer::SString::StrikeThrough ).contains( key ) )
		toggleStrikeOut();
	else
		QTextEdit::keyPressEvent( event );
}

void SimpleRichTextEdit::keyReleaseEvent( QKeyEvent* event )
{
	QTextEdit::keyReleaseEvent( event );
}

void SimpleRichTextEdit::focusInEvent( QFocusEvent* event )
{
	QTextEdit::focusInEvent( event );

	emit focusChanged( true );
}

void SimpleRichTextEdit::focusOutEvent( QFocusEvent* event )
{
	QTextEdit::focusOutEvent( event );

	emit focusChanged( false );
}

#include "simplerichtextedit.moc"
