#ifndef SIMPLERICHTEXTEDIT_H
#define SIMPLERICHTEXTEDIT_H

/***************************************************************************
*   Copyright (C) 2007-2008 by Sergio Pistone                             *
*   sergio_pistone@yahoo.com.ar                                           *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <sstring.h>

#include <qtextedit.h>

#include <kshortcut.h>

class SimpleRichTextEdit : public QTextEdit
{
	Q_OBJECT

	public:

		SimpleRichTextEdit( QWidget* parent=0 );
		virtual ~SimpleRichTextEdit();

		bool strikeOut();

	public slots:

		SubtitleComposer::SString richText();
		void setRichText( const SubtitleComposer::SString& richText );

		void setSelection( int startIndex, int endIndex );

		void setStrikeOut( bool enabled );

		void setTextFormat( TextFormat format ); // redefined to do nothing

		void toggleBold();
		void toggleItalic();
		void toggleUnderline();
		void toggleStrikeOut();

		void toggleTabChangesFocus();

	signals:

		void cursorChanged2();
		void selectionChanged2();

		void focusChanged( bool hasFocus );

	protected slots:

		void onCursorPositionChanged();
		void onSelectionChanged();

	protected:

		void setCursorPosition( int para, int index, bool emitSignals );
		void setSelection( int paraFrom, int indexFrom, int paraTo, int indexTo, int selNum, bool emitSignals );

		KShortcut getKShortcut( SubtitleComposer::SString::StyleFlag style );
		QKeySequence getQKeySequence( SubtitleComposer::SString::StyleFlag style );

		virtual QPopupMenu* createPopupMenu( const QPoint &pos );

		virtual void keyPressEvent( QKeyEvent* event );
		virtual void keyReleaseEvent( QKeyEvent* event );

		virtual void focusInEvent( QFocusEvent* event );
		virtual void focusOutEvent( QFocusEvent* event );

		bool m_emitSignals;
};

#endif
